Method: RIO::RL::WithPath#fspath
- Defined in:
- lib/rio/rl/withpath.rb
#fspath ⇒ Object
returns the path as the file system sees it. Spaces are spaces and not %20 etc. This is the path that would be passed to the fs object. For windows RLs this includes the ‘//host’ part and the ‘C:’ part returns a String
121 122 123 |
# File 'lib/rio/rl/withpath.rb', line 121 def fspath() RL.url2fs(self.urlpath) end |